
#include <Arduino.h>
#include <BLEDevice.h>
#include <BLEUtils.h>
#include <BLEServer.h>

#define GAS_SENSOR_PIN 34
#define LED_GREEN 14
#define LED_YELLOW 27
#define LED_RED 26

BLECharacteristic *pCharacteristic;
bool deviceConnected = false;

#define SERVICE_UUID        "4fafc201-1fb5-459e-8fcc-c5c9c331914b"
#define CHARACTERISTIC_UUID "beb5483e-36e1-4688-b7f5-ea07361b26a8"

class MyServerCallbacks: public BLEServerCallbacks {
    void onConnect(BLEServer* pServer) {
      deviceConnected = true;
    };

    void onDisconnect(BLEServer* pServer) {
      deviceConnected = false;
    }
};

int readGasSensor() {
  int raw = analogRead(GAS_SENSOR_PIN);
  return map(raw, 0, 4095, 0, 100); // normalize to percentage
}

void updateLED(int score) {
  digitalWrite(LED_GREEN, score > 70 ? HIGH : LOW);
  digitalWrite(LED_YELLOW, (score <= 70 && score > 40) ? HIGH : LOW);
  digitalWrite(LED_RED, score <= 40 ? HIGH : LOW);
}

void setup() {
  Serial.begin(115200);
  pinMode(GAS_SENSOR_PIN, INPUT);
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_YELLOW, OUTPUT);
  pinMode(LED_RED, OUTPUT);

  BLEDevice::init("SmartSpoilCompact");
  BLEServer *pServer = BLEDevice::createServer();
  pServer->setCallbacks(new MyServerCallbacks());

  BLEService *pService = pServer->createService(SERVICE_UUID);
  pCharacteristic = pService->createCharacteristic(
                     CHARACTERISTIC_UUID,
                     BLECharacteristic::PROPERTY_READ |
                     BLECharacteristic::PROPERTY_NOTIFY
                   );
  pCharacteristic->setValue("Freshness: 100");
  pService->start();
  BLEAdvertising *pAdvertising = BLEDevice::getAdvertising();
  pAdvertising->addServiceUUID(SERVICE_UUID);
  pAdvertising->start();
}

void loop() {
  int freshness = readGasSensor();
  String freshnessStr = "Freshness: " + String(freshness);
  pCharacteristic->setValue(freshnessStr.c_str());
  pCharacteristic->notify();
  updateLED(freshness);
  Serial.println(freshnessStr);
  delay(5000);
}
